package junit.log4j;
import org.apache.log4j.helpers.PatternParser;
import org.apache.log4j.helpers.PatternConverter;
import org.apache.log4j.helpers.FormattingInfo;
import org.apache.log4j.spi.LoggingEvent;
import java.io.PrintWriter;
import java.io.StringWriter;
/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.<p>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.<p>
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA<br>
 * http://www.gnu.org/copyleft/lesser.html
 */

// Configuration Management Information: 
// -------------------------------------
// $Id: JUnitPatternParser.java,v 1.2 2001/11/24 18:52:58 wreissen Exp $
//
// Version History:
// ----------------
// $Log: JUnitPatternParser.java,v $
// Revision 1.2  2001/11/24 18:52:58  wreissen
// convert uses event.getThrowableStrRep() instead of
// an own implementation of stacktrace --> String
//
// Revision 1.1  2001/11/17 22:38:46  wreissen
// initial version.
//
//
// ***********************************************************************************
/**
 * <p>Extension of {@link org.apache.log4j.helpers.PatternParser}
 * writing the stack trace of the throwable to the result string.</p>
 *
 *
 * Created: Sat Nov 17 15:19:09 2001
 *
 * @author <a href="mailto:wolfgang@openfuture.de">Wolfgang Reissenberger</a>
 * @version $Revision: 1.2 $
 */

public class JUnitPatternParser extends PatternParser {

    /**
     * The conversion character <b>T</b> stands for stack traces. 
     *
     */
    public static final char STACKTRACE_CHAR = 'T';
    
    /**
     * Creates a new <code>JUnitPatternParser</code> instance.
     *
     * @param pattern the supplied conversion pattern
     */
    public JUnitPatternParser(String pattern) {
	super(pattern);
    }
    

    /**
     * In case of {@link #STACKTRACE_CHAR T}, the stack trace
     * is written. Otherwise, the character is delegated to the
     * super class.
     *
     * @param formatChar the format character
     */
    public void finalizeConverter(char formatChar) {

       PatternConverter pc = null;
       switch( formatChar ) {
          case STACKTRACE_CHAR:

             pc = new StacktracePatternConverter(formattingInfo);

             currentLiteral.setLength(0);
             addConverter(pc);
             break;

          default:
             super.finalizeConverter( formatChar );

       }
    }

    /**
     * Converter writing the stack trace of the throwable.
     *
     */
    public static class StacktracePatternConverter
	extends PatternConverter {

	/**
	 * Creates a new <code>StacktracePatternConverter</code> instance.
	 *
	 * @param formattingInfo a <code>FormattingInfo</code> value
	 */
	public StacktracePatternConverter(FormattingInfo formattingInfo) {
	    super(formattingInfo);     
	}

	/**
	 * writing the stack trace of the throwable contained in
	 * <code>event</code>.
	 *
	 * @param event a <code>LoggingEvent</code> value
	 * @return <code>null</code>, if the event is null or its
	 * {@link org.apache.log4j.spi.ThrowableInformation#getThrowable()
	 *        throwable}. Otherwise, a string representation of the
	 * stack trace is returned.
	 */
	public String convert(LoggingEvent event) {
	    if (event == null) return null;
	    if (event.getThrowableInformation() == null) return null;
	    String[] results = event.getThrowableStrRep();

	    if (results != null && results.length > 0) {
		StringBuffer result = new StringBuffer();

		result.append(results[0]);
		for (int i = 1; i < results.length; i++) {
		    result.append("\n");
		    result.append(results[i]);
		}

		return result.toString();
	    } else return null;
	}
	
    } // StacktracePatternConverter
} // JUnitPatternParser
